@@ -60,6 +60,7 @@
 #include <KToggleAction>
 #include <KLocalizedString>
 #include <KXMLGUIFactory>
+#include <kparts_version.h>
 
 #include <QAction>
 #include <QCursor>
@@ -776,11 +777,12 @@ bool Part::openFile()
 
     if (creatingNewArchive) {
         createArchive();
-    } else {
-        loadArchive();
+        return true;
     }
 
-    return true;
+    loadArchive();
+    // Loading is async, we don't know yet whether we got a valid archive.
+    return false;
 }
 
 bool Part::saveFile()
@@ -890,7 +892,15 @@ void Part::slotLoadingFinished(KJob *job)
             m_infoPanel->updateWithDefaults();
 
             emit setWindowCaption(QString());
+#if KPARTS_VERSION >= QT_VERSION_CHECK(5, 37, 0)
+            // See https://phabricator.kde.org/D6856
+            closeUrl();
+#else
+            setUrl(QUrl());
+#endif
         }
+    } else {
+        emit completed();
     }
 
     m_view->sortByColumn(0, Qt::AscendingOrder);
